@@ -26,7 +26,7 @@ dependencies using raw pip. The virtual environment is not created, but one can
 issue `python3 -m venv venv/ && . venv/bin/activate` to create one.
 """
 
-__version__ = "0.1.4"
+__version__ = "0.1.6"
 __author__ = "Fridolin Pokorny <fridex.devel@gmail.com>"
 __title__ = "micropipenv"
 __all__ = [
@@ -89,6 +89,7 @@ except ImportError:
 if TYPE_CHECKING:
     from typing import Any
     from typing import Dict
+    from typing import Generator
     from typing import List
     from typing import MutableMapping
     from typing import Optional
@@ -273,8 +274,6 @@ def install_pipenv(
                 "Pipfile {!r}, aborting deployment".format(pipfile_lock_hash, pipfile_hash)
             )
 
-    index_config_str = _get_index_entry_str(sections)
-
     tmp_file = tempfile.NamedTemporaryFile("w", prefix="requirements.txt", delete=False)
     _LOGGER.debug("Using temporary file for storing requirements: %r", tmp_file.name)
 
@@ -295,18 +294,28 @@ def install_pipenv(
             entry = to_install.popleft()
             package_name, info, had_error = entry["package_name"], entry["info"], entry["error"]
 
-            with open(tmp_file.name, "w") as f:
-                f.write(index_config_str)
-                package_entry_str = _get_package_entry_str(package_name, info)
-                f.write(package_entry_str)
-
             if "git" in info:
                 _LOGGER.warning("!!! Requirement %s uses a VCS version: %r", package_name, info)
 
-            _LOGGER.info("Installing %r", package_entry_str)
-            called_process = subprocess.run(cmd)
+            package_entry_str = _get_package_entry_str(package_name, info)
+            for index_config_str in _iter_index_entry_str(sections, info):
+                with open(tmp_file.name, "w") as f:
+                    f.write(index_config_str)
+                    f.write(package_entry_str)
 
-            if called_process.returncode != 0:
+                _LOGGER.info("Installing %r", package_entry_str)
+                called_process = subprocess.run(cmd)
+
+                if called_process.returncode == 0:
+                    # Discard any error flag if we have any packages that fail to install.
+                    for item in reversed(to_install):
+                        if item["error"] == 0:
+                            # Fast path - errors are always added to the end of the queue,
+                            # if we find a package without any error we can safely break.
+                            break
+                        item["error"] = 0
+                    break
+            else:
                 if len(to_install) == 0 or (had_error and had_error > len(to_install)):
                     raise PipInstallError(
                         "Failed to install requirements, dependency {!r} could not be installed".format(package_name)
@@ -314,14 +323,7 @@ def install_pipenv(
 
                 _LOGGER.warning("Failed to install %r, will try in next installation round...", package_name)
                 to_install.append({"package_name": package_name, "info": info, "error": had_error + 1})
-            else:
-                # Discard any error flag if we have any packages that fail to install.
-                for item in reversed(to_install):
-                    if item["error"] == 0:
-                        # Fast path - errors are always added to the end of the queue,
-                        # if we find a package without any error we can safely break.
-                        break
-                    item["error"] = 0
+
     finally:
         os.remove(tmp_file.name)
 
@@ -353,32 +355,31 @@ def _requirements2pipfile_lock():  # type: () -> Dict[str, Any]
 
     result = {}  # type: Dict[str, Any]
     for requirement in parse_requirements(filename=requirements_txt_path, session=PipSession(), finder=finder):
-        version = str(requirement.specifier)
+        entry = {}  # type: Dict[str, Any]
 
-        if requirement.specifier is None or not (
-            requirement.has_hash_options and len(requirement.specifier) == 1 and version.startswith("==")
-        ):
-            # Not pinned down software stack using pip-tools.
-            raise PipRequirementsNotLocked
+        if not requirement.editable:
+            version = str(requirement.specifier)
 
-        # We add all dependencies to default, develop should not be present in requirements.txt file, but rather
-        # in dev-requirements.txt or similar.
-        if requirement.name in result:
-            raise RequirementsError("Duplicate entry for requirement {}".format(requirement.name))
+            if requirement.specifier is None or not (
+                requirement.has_hash_options and len(requirement.specifier) == 1 and version.startswith("==")
+            ):
+                # Not pinned down software stack using pip-tools.
+                raise PipRequirementsNotLocked
 
-        if hasattr(requirement, "options"):
-            hash_options = requirement.options.get("hashes")
+            if hasattr(requirement, "options"):
+                hash_options = requirement.options.get("hashes")
+            else:
+                hash_options = requirement.hash_options  # More recent pip.
+
+            hashes = []
+            for hash_type, hashes_present in hash_options.items():
+                hashes.extend(["{}:{}".format(hash_type, h) for h in hashes_present])
+
+            entry["hashes"] = sorted(hashes)
+            entry["version"] = version
         else:
-            hash_options = requirement.hash_options  # More recent pip.
-
-        hashes = []
-        for hash_type, hashes_present in hash_options.items():
-            hashes.extend(["{}:{}".format(hash_type, h) for h in hashes_present])
-
-        entry = {
-            "hashes": sorted(hashes),
-            "version": version,
-        }
+            entry["editable"] = True
+            entry["path"] = str(requirement.link)
 
         if requirement.extras:
             entry["extras"] = sorted(requirement.extras)
@@ -386,21 +387,38 @@ def _requirements2pipfile_lock():  # type: () -> Dict[str, Any]
         if requirement.markers:
             entry["markers"] = str(requirement.markers)
 
-        result[requirement.name] = entry
+        if entry.get("editable", False):
+            # Create a unique name for editable to avoid possible clashes.
+            requirement_name = hashlib.sha256(json.dumps(entry, sort_keys=True).encode("utf8")).hexdigest()
+        else:
+            requirement_name = requirement.name
+
+        # We add all dependencies to default, develop should not be present in requirements.txt file, but rather
+        # in dev-requirements.txt or similar.
+        if requirement_name in result:
+            raise RequirementsError("Duplicate entry for requirement {}".format(requirement.name))
+
+        result[requirement_name] = entry
+
+    if all(dep.get("editable", False) for dep in result.values()):
+        # If all the dependencies are editable, we cannot safely say that we
+        # have a lock file - users can easily end up with missing dependencies
+        # as we install dependencies with --no-deps in case of lock files. Let
+        # pip resolver do its job, just to be sure.
+        raise PipRequirementsNotLocked
 
     sources = []  # type: List[Dict[str, Any]]
     for index_url in chain(finder.index_urls, _DEFAULT_INDEX_URLS):
         if any(s["url"] == index_url for s in sources):
             continue
 
-        sources.append(
-            {"name": hashlib.sha256(index_url.encode()).hexdigest(), "url": index_url, "verify_ssl": True}
-        )
+        sources.append({"name": hashlib.sha256(index_url.encode()).hexdigest(), "url": index_url, "verify_ssl": True})
 
     if len(sources) == 1:
         # Explicitly assign index if there is just one.
         for entry in result.values():
-            entry["index"] = sources[0]["name"]
+            if not entry.get("editable", False):
+                entry["index"] = sources[0]["name"]
 
     with open(requirements_txt_path, "r") as requirements_file:
         requirements_hash = hashlib.sha256(requirements_file.read().encode()).hexdigest()
@@ -470,9 +488,7 @@ def _poetry2pipfile_lock(
     sources = []
     has_default = False  # If default flag is set, it disallows PyPI.
     for item in pyproject_poetry_section.get("source", []):
-        sources.append(
-            {"name": item["name"], "url": item["url"], "verify_ssl": True}
-        )
+        sources.append({"name": item["name"], "url": item["url"], "verify_ssl": True})
 
         has_default = has_default or item.get("default", False)
 
@@ -632,9 +648,15 @@ def install(
                 _traverse_up_find_file("poetry.lock")
                 method = "poetry"
             except FileNotFound as exc:
-                _LOGGER.info("Failed to find Poetry lock file, looking up requirements.txt: %s", str(exc))
-                _traverse_up_find_file("requirements.txt")
-                method = "requirements"
+                try:
+                    _LOGGER.info("Failed to find Poetry lock file, looking up requirements.txt: %s", str(exc))
+                    _traverse_up_find_file("requirements.txt")
+                    method = "requirements"
+                except FileNotFound as exc:
+                    raise FileNotFound(
+                        "Failed to find Pipfile.lock, poetry.lock or requirements.txt "
+                        "in the current directory or any of its parent: {}".format(os.getcwd())
+                    ) from exc
 
     if method == "requirements":
         if deploy:
@@ -728,13 +750,21 @@ def _get_package_entry_str(
 ):  # type: (str, Dict[str, Any], bool, bool) -> str
     """Print entry for the given package."""
     if "git" in info:  # A special case for a VCS package
-        result = "git+{}".format(info["git"])
+        if info.get("editable", False):
+            result = "--editable git+{}".format(info["git"])
+        else:
+            result = "git+{}".format(info["git"])
+
         if "ref" in info:
             result += "@{}".format(info["ref"])
-        result += "#egg={}".format(package_name)
+        result += "#egg={}\n".format(package_name)
         return result
 
-    result = package_name
+    if info.get("editable", False):
+        result = "--editable {}".format(info.get("path", "."))
+    else:
+        result = package_name
+
     if info.get("extras"):
         result += "[{}]".format(",".join(info["extras"]))
 
@@ -744,7 +774,7 @@ def _get_package_entry_str(
     if info.get("markers"):
         result += "; {}".format(info["markers"])
 
-    if not (no_hashes or no_versions):
+    if not (no_hashes or no_versions or info.get("editable", False)):
         for digest in info.get("hashes", []):
             result += " \\\n"
             result += "    --hash={}".format(digest)
@@ -752,19 +782,55 @@ def _get_package_entry_str(
     return result + "\n"
 
 
-def _get_index_entry_str(sections):  # type: (Dict[str, Any]) -> str
+def _get_index_entry_str(sections, package_info=None):  # type: (Dict[str, Any], Optional[Dict[str, Any]]) -> str
     """Get configuration entry for Python package indexes."""
+    index_name = package_info.get("index") if package_info is not None else None
+
     result = ""
     for idx, source in enumerate(sections.get("sources", [])):
-        if idx == 0:
-            result += "--index-url {}\n".format(source["url"])
-        else:
-            result += "--extra-index-url {}\n".format(source["url"])
+        if index_name is None:
+            if idx == 0:
+                result += "--index-url {}\n".format(source["url"])
+            else:
+                result += "--extra-index-url {}\n".format(source["url"])
 
+            if not source["verify_ssl"]:
+                result += "--trusted-host {}\n".format(urlparse(source["url"]).netloc)
+        else:
+            if index_name == source["name"]:
+                result += "--index-url {}\n".format(source["url"])
+
+                if not source["verify_ssl"]:
+                    result += "--trusted-host {}\n".format(urlparse(source["url"]).netloc)
+
+                break
+
+    if index_name is not None and not result:
+        raise RequirementsError(
+            "No index found given the configuration: {} (package info: {})".format(sections, package_info)
+        )
+
+    return result
+
+
+def _iter_index_entry_str(
+    sections, package_info
+):  # type: (Dict[str, Any], Optional[Dict[str, Any]]) -> Generator[str, None, None]
+    """Iterate over possible package index configurations for the given package to try all possible installations."""
+    index_name = package_info.get("index") if package_info is not None else None
+
+    if index_name is not None or len(sections.get("sources") or []) <= 1:
+        # No need to iterate over indexes, the package index is set and is just one.
+        res = _get_index_entry_str(sections, package_info)
+        yield from [res]
+        return None
+
+    for source in sections["sources"]:
+        result = "--index-url {}\n".format(source["url"])
         if not source["verify_ssl"]:
             result += "--trusted-host {}\n".format(urlparse(source["url"]).netloc)
 
-    return result
+        yield result
 
 
 def requirements_str(
@@ -867,7 +933,7 @@ def main(argv=None):  # type: (Optional[List[str]]) -> int
         "--method",
         help="Source of packages for the installation, perform detection if not provided.",
         choices=["pipenv", "requirements", "poetry"],
-        default=os.getenv("MICROPIPENV_METHOD")
+        default=os.getenv("MICROPIPENV_METHOD"),
     )
     parser_install.add_argument(
         "pip_args",
@@ -933,7 +999,7 @@ def main(argv=None):  # type: (Optional[List[str]]) -> int
         "--method",
         help="Source of packages for the requirements file, perform detection if not provided.",
         choices=["pipenv", "poetry"],
-        default=os.getenv("MICROPIPENV_METHOD", "pipenv")
+        default=os.getenv("MICROPIPENV_METHOD", "pipenv"),
     )
     parser_requirements.set_defaults(func=requirements)
 
